/**
 * This is an interface for customizing the classes for
 * the `BulkCheckboxControl` component.
 */
export interface BulkCheckboxControlClasses {
    /**
     * The class for the "Select all" or "Deselect all" text.
     * This defaults to `link-primary text-decoration-none`.
     */
    selectRemoveAllText?: string;
}
/**
 * This is an interface for `BulkCheckboxControl` component props.
 */
export interface BulkCheckboxControlProps {
    /** Custom classes for the component. */
    classes?: BulkCheckboxControlClasses;
}
/**
 * Renders a control for selection or deselection of all rows. This is
 * only useful when pagination and checkbox are both enabled. When only
 * some (or none) of the rows are checked, then this will render the
 * number of checked rows, as well as the "Select all" button.
 * On the other hand, when all rows are selected, then it will
 * change to "Deselect all" button.
 *
 * This is only fit for an uncontrolled table.
 */
export declare function BulkCheckboxControl({ classes }: BulkCheckboxControlProps): JSX.Element;
